# -*- coding: utf-8 -*-

from pinba.defaults import (
    PINBA_SERVER, PINBA_PORT, PINBA_ENABLED, PINBA_DISABLE_ADMIN
)
from pinba.monitor import Monitor


class PinbaMiddleware(object):
    def __init__(self):
        super(PinbaMiddleware, self).__init__()
        self.monitor = Monitor((PINBA_SERVER, PINBA_PORT))

    @staticmethod
    def report_is_enabled(request):
        if not PINBA_ENABLED:
            return False

        if PINBA_DISABLE_ADMIN is True:
            if hasattr(request, 'user') and hasattr(request.user, 'is_staff'):
                if request.user.is_staff:
                    return False
        return True

    def process_request(self, request):
        if self.report_is_enabled(request):
            self.monitor.start(request)

    def process_response(self, request, response):
        if self.report_is_enabled(request):
            return self.monitor.stop(response)
        return response



# -*- coding: utf-8 -*-
"""
    IsCool-e Pynba
    ~~~~~~~~~~~~~~

    :copyright: (c) 2012 by IsCool Entertainment.
    :license: MIT, see LICENSE for more details.
"""

import logging
import socket
from pinba.message import dumps

logger = logging.getLogger('pinba')


class Reporter(object):
    def __init__(self, address, raise_on_fail=False):
        self.address = address
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.raise_on_fail = raise_on_fail

    def __call__(self, server_name, hostname, script_name,
                 elapsed, ru_utime=0.0, ru_stime=0.0,
                 document_size=0, memory_peak=0, status=200,
                 memory_footprint=0, schema='http'):

        self.send(dumps(**{
            'hostname': hostname,
            'server_name': server_name,
            'script_name': script_name,
            'request_count': 1,
            'document_size': document_size,
            'memory_peak': memory_peak,
            'request_time': elapsed,
            'ru_utime': ru_utime,
            'ru_stime': ru_stime,
            'status': status,
            'memory_footprint': memory_footprint,
            'schema': schema,
        }))

    def send(self, msg):
        try:
            return self.sock.sendto(msg, self.address)
        except Exception as error:
            if self.raise_on_fail:
                raise
            logger.exception(error)
        return None



# -*- coding: utf-8 -*-
"""
    IsCool-e Pynba
    ~~~~~~~~~~~~~~

    :copyright: (c) 2012 by IsCool Entertainment.
    :license: MIT, see LICENSE for more details.
"""

from six import BytesIO, PY3
from collections import namedtuple
import struct

Field = namedtuple('Field', 'tag name type rule')
fields = [
    Field(1, 'hostname', 'string', 'required'),
    Field(2, 'server_name', 'string', 'required'),
    Field(3, 'script_name', 'string', 'required'),
    Field(4, 'request_count', 'uint32', 'required'),
    Field(5, 'document_size', 'uint32', 'required'),
    Field(6, 'memory_peak', 'uint32', 'required'),
    Field(7, 'request_time', 'float', 'required'),
    Field(8, 'ru_utime', 'float', 'required'),
    Field(9, 'ru_stime', 'float', 'required'),
    Field(10, 'timer_hit_count', 'uint32', 'repeated'),
    Field(11, 'timer_value', 'float', 'repeated'),
    Field(12, 'timer_tag_count', 'uint32', 'repeated'),
    Field(13, 'timer_tag_name', 'uint32', 'repeated'),
    Field(14, 'timer_tag_value', 'uint32', 'repeated'),
    Field(15, 'dictionary', 'string', 'repeated'),
    Field(16, 'status', 'uint32', 'optional'),
    Field(17, 'memory_footprint', 'uint32', 'optional'),
    Field(18, 'requests', 'Request', 'repeated'),
    Field(19, 'schema', 'string', 'optional'),
    Field(20, 'tag_name', 'uint32', 'repeated'),
    Field(21, 'tag_value', 'uint32', 'repeated'),
]


def pack_tag(tag, wire_type):
    return (tag << 3) | wire_type


def write_uvarint(file, value):
    shifted_value = True
    while shifted_value:
        shifted_value = value >> 7
        file.write(cast(chr(
            (value & 0x7F) | (0x80 if shifted_value != 0 else 0x00))))  # noqa
        value = shifted_value


def write_float(file, value):
    packed_value = struct.pack('<f', value)
    file.write(packed_value)


def write_string(file, value):
    value = cast(value)
    write_uvarint(file, len(value))
    file.write(value)


def write_tag(file, tag, wire_type):
    write_uvarint(file, pack_tag(tag, wire_type))


def dumps(**attrs):
    file = BytesIO()

    loop = (field for field in fields if field.name in attrs)
    for field in loop:
        buffer = attrs[field.name]

        if field.rule in ('repeated',):
            values = buffer
        else:
            values = [buffer]

        for value in values:
            if field.type == 'uint32':
                wire_type = 0
                write_tag(file, field.tag, wire_type)
                write_uvarint(file, value)
                continue
            elif field.type == 'float':
                wire_type = 5
                write_tag(file, field.tag, wire_type)
                write_float(file, value)
                continue
            elif field.type == 'string':
                wire_type = 2
                write_tag(file, field.tag, wire_type)
                write_string(file, value)
                continue
            raise NotImplementedError(field)

    return file.getvalue()


if PY3:
    def cast(value):
        """Cast any value to bytes()"""
        if isinstance(value, str):
            return value.encode('utf-8')
        if not isinstance(value, bytes):
            return bytes(value, 'utf-8')
        return value
else:
    def cast(value):
        """Cast any value to bytes()"""
        if isinstance(value, unicode):
            return value.encode('utf-8')
        if not isinstance(value, str):
            return str(value)
        return value



# -*- coding: utf-8 -*-

import resource
import socket
import time

from django.core.urlresolvers import resolve

from pinba.reporter import Reporter
from pinba import defaults


class Monitor(object):
    def __init__(self, address):
        self.hostname = socket.gethostname()
        self.reporter = Reporter(address)
        self.document_size = None
        self.memory_peak = None
        self.script_name = None
        self.server_name = None
        self.start_time = None
        self.resources = None
        self.ru_utime = None
        self.ru_stime = None
        self.response = None
        self.request = None
        self.elapsed = None
        self.usage = None
        self.status = 200

    def get_server_name(self):
        if defaults.PINBA_SERVER_NAME is not None:
            return defaults.PINBA_SERVER_NAME
        return self.request.META.get('SERVER_NAME')

    def start(self, request):
        self.resources = resource.getrusage(resource.RUSAGE_SELF)
        self.start_time = time.time()
        self.request = request

    def stop(self, response):
        self.usage = resource.getrusage(resource.RUSAGE_SELF)
        self.elapsed = (time.time() - self.start_time)
        self.response = response

        self._calculate()
        self._send()
        return response

    def _calculate(self):
        self.ru_utime = self.usage.ru_utime - self.resources.ru_utime
        self.ru_stime = self.usage.ru_stime - self.resources.ru_stime
        self.server_name = self.get_server_name()
        self.document_size = len(self.response.content)
        self.script_name = self._get_script_name()
        self.memory_peak = self.usage.ru_maxrss
        self.status = self.response.status_code

    def _get_script_name(self):
        try:
            current_view = resolve(self.request.path)[0]
            return '%s.%s' % (current_view.__module__, current_view.__name__)
        except:
            return self.request.path

    def _get_scheme(self):
        if hasattr(self.request, 'scheme'):
            return self.request.scheme
        return 'http'

    def _send(self):
        self.reporter(
            self.server_name,
            self.hostname,
            self.script_name,
            self.elapsed,
            document_size=self.document_size,
            memory_peak=self.memory_peak,
            ru_utime=self.ru_utime,
            ru_stime=self.ru_stime,
            status=self.status,
            schema=self._get_scheme(),
            # memory_footprint=self.memory_peak,
        )



# -*- coding: utf-8 -*-

from django.conf import settings

PINBA_SERVER = getattr(settings, 'PINBA_SERVER', '127.0.0.1')
PINBA_PORT = getattr(settings, 'PINBA_PORT', 30002)
PINBA_ENABLED = getattr(settings, 'PINBA_ENABLED', False)
PINBA_SERVER_NAME = getattr(settings, 'PINBA_SERVER_NAME', None)
PINBA_DISABLE_ADMIN = getattr(settings, 'PINBA_DISABLE_ADMIN', False)
